@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2015 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -13,8 +13,7 @@ import os
 
 from PyQt5.QtCore import pyqtSignal, QObject, Qt
 from PyQt5.QtGui import QKeySequence
-from PyQt5.QtWidgets import QMenu, QToolBar, QApplication, QDialog, \
-    QInputDialog
+from PyQt5.QtWidgets import QMenu, QToolBar, QApplication, QDialog
 
 from UI.Info import Program
 
@@ -104,15 +103,13 @@ class DebugUI(QObject):
         self.forkIntoChild = Preferences.toBool(
             Preferences.Prefs.settings.value('DebugInfo/ForkIntoChild', False))
         
-        self.evalHistory = []
-        self.execHistory = []
         self.lastDebuggedFile = None
         self.lastStartAction = 0    # 0=None, 1=Script, 2=Project
         self.clientType = ""
         self.lastAction = -1
         self.debugActions = [
             self.__continue, self.__step, self.__stepOver, self.__stepOut,
-            self.__stepQuit, self.__runToCursor
+            self.__stepQuit, self.__runToCursor, self.__moveInstructionPointer
         ]
         self.localsVarFilter, self.globalsVarFilter = \
             Preferences.getVarFilters()
@@ -360,6 +357,26 @@ class DebugUI(QObject):
         ))
         act.triggered.connect(self.__runToCursor)
         self.actions.append(act)
+        
+        act = E5Action(
+            self.tr('Move Instruction Pointer to Cursor'),
+            UI.PixmapCache.getIcon("moveInstructionPointer.png"),
+            self.tr('&Jump To Cursor'), Qt.Key_F12, 0,
+            self.debugActGrp, 'dbg_jump_to_cursor')
+        act.setStatusTip(self.tr(
+            """Skip the code from the"""
+            """ current line to the current cursor position"""))
+        act.setWhatsThis(self.tr(
+            """<b>Move Instruction Pointer to Cursor</b>"""
+            """<p>Move the Python internal instruction pointer to the"""
+            """ current cursor position without executing the code in"""
+            """ between.</p>"""
+            """<p>It's not possible to jump out of a function or jump"""
+            """ in a code block, e.g. a loop. In these cases, a error"""
+            """ message is printed to the log window.</p>"""
+        ))
+        act.triggered.connect(self.__moveInstructionPointer)
+        self.actions.append(act)
 
         act = E5Action(
             self.tr('Single Step'),
@@ -428,36 +445,6 @@ class DebugUI(QObject):
         act.triggered.connect(self.__stepQuit)
         self.actions.append(act)
         
-        self.debugActGrp2 = createActionGroup(self)
-
-        act = E5Action(
-            self.tr('Evaluate'),
-            self.tr('E&valuate...'),
-            0, 0, self.debugActGrp2, 'dbg_evaluate')
-        act.setStatusTip(self.tr('Evaluate in current context'))
-        act.setWhatsThis(self.tr(
-            """<b>Evaluate</b>"""
-            """<p>Evaluate an expression in the current context of the"""
-            """ debugged program. The result is displayed in the"""
-            """ shell window.</p>"""
-        ))
-        act.triggered.connect(self.__eval)
-        self.actions.append(act)
-        
-        act = E5Action(
-            self.tr('Execute'),
-            self.tr('E&xecute...'),
-            0, 0, self.debugActGrp2, 'dbg_execute')
-        act.setStatusTip(
-            self.tr('Execute a one line statement in the current context'))
-        act.setWhatsThis(self.tr(
-            """<b>Execute</b>"""
-            """<p>Execute a one line statement in the current context"""
-            """ of the debugged program.</p>"""
-        ))
-        act.triggered.connect(self.__exec)
-        self.actions.append(act)
-        
         self.dbgFilterAct = E5Action(
             self.tr('Variables Type Filter'),
             self.tr('Varia&bles Type Filter...'), 0, 0, self,
@@ -585,7 +572,6 @@ class DebugUI(QObject):
         self.actions.append(act)
 
         self.debugActGrp.setEnabled(False)
-        self.debugActGrp2.setEnabled(False)
         self.dbgSetBpActGrp.setEnabled(False)
         self.runAct.setEnabled(False)
         self.runProjectAct.setEnabled(False)
@@ -627,8 +613,6 @@ class DebugUI(QObject):
         
         dmenu.addActions(self.debugActGrp.actions())
         dmenu.addSeparator()
-        dmenu.addActions(self.debugActGrp2.actions())
-        dmenu.addSeparator()
         dmenu.addActions(self.dbgSetBpActGrp.actions())
         self.menuBreakpointsAct = dmenu.addMenu(self.breakpointsMenu)
         dmenu.addSeparator()
@@ -686,50 +670,59 @@ class DebugUI(QObject):
         
         return [starttb, debugtb]
         
-    def setArgvHistory(self, argsStr, clearHistories=False):
+    def setArgvHistory(self, argsStr, clearHistories=False, history=None):
         """
         Public slot to initialize the argv history.
         
         @param argsStr the commandline arguments (string)
         @param clearHistories flag indicating, that the list should
             be cleared (boolean)
+        @param history list of history entries to be set (list of strings)
         """
         if clearHistories:
-            self.argvHistory = []
+            del self.argvHistory[1:]
+        elif history is not None:
+            self.argvHistory = history[:]
         else:
             if argsStr in self.argvHistory:
                 self.argvHistory.remove(argsStr)
-        self.argvHistory.insert(0, argsStr)
+            self.argvHistory.insert(0, argsStr)
 
-    def setWdHistory(self, wdStr, clearHistories=False):
+    def setWdHistory(self, wdStr, clearHistories=False, history=None):
         """
         Public slot to initialize the wd history.
         
         @param wdStr the working directory (string)
         @param clearHistories flag indicating, that the list should
             be cleared (boolean)
+        @param history list of history entries to be set (list of strings)
         """
         if clearHistories:
-            self.wdHistory = []
+            del self.wdHistory[1:]
+        elif history is not None:
+            self.wdHistory = history[:]
         else:
             if wdStr in self.wdHistory:
                 self.wdHistory.remove(wdStr)
-        self.wdHistory.insert(0, wdStr)
+            self.wdHistory.insert(0, wdStr)
         
-    def setEnvHistory(self, envStr, clearHistories=False):
+    def setEnvHistory(self, envStr, clearHistories=False, history=None):
         """
         Public slot to initialize the env history.
         
         @param envStr the environment settings (string)
         @param clearHistories flag indicating, that the list should
             be cleared (boolean)
+        @param history list of history entries to be set (list of strings)
         """
         if clearHistories:
-            self.envHistory = []
+            del self.envHistory[1:]
+        elif history is not None:
+            self.envHistory = history[:]
         else:
             if envStr in self.envHistory:
                 self.envHistory.remove(envStr)
-        self.envHistory.insert(0, envStr)
+            self.envHistory.insert(0, envStr)
         
     def setExceptionReporting(self, exceptions):
         """
@@ -806,7 +799,6 @@ class DebugUI(QObject):
         self.profileAct.setEnabled(False)
         self.coverageAct.setEnabled(False)
         self.debugActGrp.setEnabled(False)
-        self.debugActGrp2.setEnabled(False)
         self.dbgSetBpActGrp.setEnabled(False)
         self.lastAction = -1
         if not self.projectOpen:
@@ -891,7 +883,7 @@ class DebugUI(QObject):
         """
         self.projectOpen = True
         cap = self.debugServer.getClientCapabilities(
-            self.project.pdata["PROGLANGUAGE"][0])
+            self.project.getProjectLanguage())
         if not self.passive:
             self.debugProjectAct.setEnabled(cap & HasDebugger)
             self.runProjectAct.setEnabled(cap & HasInterpreter)
@@ -914,6 +906,21 @@ class DebugUI(QObject):
             self.lastStartAction = 0
             self.clientType = ""
         
+    def clearHistories(self):
+        """
+        Public method to clear the various debug histories.
+        """
+        self.argvHistory = []
+        self.wdHistory = []
+        self.envHistory = []
+        
+        Preferences.Prefs.settings.setValue(
+            'DebugInfo/ArgumentsHistory', self.argvHistory)
+        Preferences.Prefs.settings.setValue(
+            'DebugInfo/WorkingDirectoryHistory', self.wdHistory)
+        Preferences.Prefs.settings.setValue(
+            'DebugInfo/EnvironmentHistory', self.envHistory)
+        
     def shutdown(self):
         """
         Public method to perform shutdown actions.
@@ -963,7 +970,6 @@ class DebugUI(QObject):
         """
         self.lastAction = -1
         self.debugActGrp.setEnabled(False)
-        self.debugActGrp2.setEnabled(False)
         if not self.passive:
             if self.editorOpen:
                 editor = self.viewmanager.activeWindow()
@@ -1001,41 +1007,53 @@ class DebugUI(QObject):
             self.__getClientVariables()
 
         self.debugActGrp.setEnabled(True)
-        self.debugActGrp2.setEnabled(True)
         
-    def __clientExit(self, status):
+    def __clientExit(self, status, message):
         """
         Private method to handle the debugged program terminating.
         
-        @param status exit code of the debugged program (int)
+        @param status exit code of the debugged program
+        @type int
+        @param message exit message of the debugged program
+        @type str
         """
         self.viewmanager.exit()
 
         self.__resetUI()
         
         if not Preferences.getDebugger("SuppressClientExit") or status != 0:
+            if message:
+                info = self.tr("<p>Message: {0}</p>").format(
+                    Utilities.html_uencode(message))
+            else:
+                info = ""
             if self.ui.currentProg is None:
                 E5MessageBox.information(
                     self.ui, Program,
                     self.tr('<p>The program has terminated with an exit'
-                            ' status of {0}.</p>').format(status))
+                            ' status of {0}.</p>{1}').format(status, info))
             else:
                 E5MessageBox.information(
                     self.ui, Program,
                     self.tr('<p><b>{0}</b> has terminated with an exit'
-                            ' status of {1}.</p>')
+                            ' status of {1}.</p>{2}')
                         .format(Utilities.normabspath(self.ui.currentProg),
-                                status))
+                                status, info))
         else:
+            if message:
+                info = self.tr("Message: {0}").format(
+                    Utilities.html_uencode(message))
+            else:
+                info = ""
             if self.ui.notificationsEnabled():
                 if self.ui.currentProg is None:
                     msg = self.tr('The program has terminated with an exit'
-                                  ' status of {0}.').format(status)
+                                  ' status of {0}.\n{1}').format(status, info)
                 else:
                     msg = self.tr('"{0}" has terminated with an exit'
-                                  ' status of {1}.')\
+                                  ' status of {1}.\n{2}')\
                         .format(os.path.basename(self.ui.currentProg),
-                                status)
+                                status, info)
                 self.ui.showNotification(
                     UI.PixmapCache.getPixmap("debug48.png"),
                     self.tr("Program terminated"), msg)
@@ -1043,13 +1061,13 @@ class DebugUI(QObject):
                 if self.ui.currentProg is None:
                     self.appendStdout.emit(
                         self.tr('The program has terminated with an exit'
-                                ' status of {0}.\n').format(status))
+                                ' status of {0}.\n{1}\n').format(status, info))
                 else:
                     self.appendStdout.emit(
                         self.tr('"{0}" has terminated with an exit'
-                                ' status of {1}.\n')
+                                ' status of {1}.\n{2}\n')
                             .format(Utilities.normabspath(self.ui.currentProg),
-                                    status))
+                                    status, info))
 
     def __clientSyntaxError(self, message, filename, lineNo, characterNo):
         """
@@ -1102,9 +1120,8 @@ class DebugUI(QObject):
         @param stackTrace list of stack entries (list of string)
         """
         self.ui.raise_()
-        self.ui.activateWindow()
         QApplication.processEvents()
-        if exceptionType is None:
+        if not exceptionType:
             E5MessageBox.critical(
                 self.ui, Program,
                 self.tr('An unhandled exception occured.'
@@ -1122,16 +1139,23 @@ class DebugUI(QObject):
                     file, line = stackTrace[0][:2]
                     source, encoding = Utilities.readEncodedFile(file)
                     source = source.splitlines(True)
-                    if len(source) >= line and \
-                       "__IGNORE_EXCEPTION__" in Utilities.extractLineFlags(
-                            source[line - 1]):
-                        res = E5MessageBox.No
+                    if len(source) >= line:
+                        lineFlags = Utilities.extractLineFlags(
+                            source[line - 1].strip())
+                        try:
+                            lineFlags += Utilities.extractLineFlags(
+                                source[line].strip(), flagsLine=True)
+                        except IndexError:
+                            pass
+                        if "__IGNORE_EXCEPTION__" in lineFlags:
+                            res = E5MessageBox.No
                 except (UnicodeError, IOError):
                     pass
                 if res != E5MessageBox.No:
                     self.viewmanager.setFileLine(
                         stackTrace[0][0], stackTrace[0][1], True)
             if res != E5MessageBox.No:
+                self.ui.activateWindow()
                 if Preferences.getDebugger("BreakAlways"):
                     res = E5MessageBox.Yes
                 else:
@@ -1177,7 +1201,6 @@ class DebugUI(QObject):
                 self.__getClientVariables()
                 self.ui.setDebugProfile()
                 self.debugActGrp.setEnabled(True)
-                self.debugActGrp2.setEnabled(True)
                 return
             elif res == E5MessageBox.Ignore:
                 if exceptionType not in self.excIgnoreList:
@@ -1282,9 +1305,9 @@ class DebugUI(QObject):
         @param variables the list of members of a classvariable from the client
         """
         if scope == 1:
-            self.debugViewer.showVariable(variables, 1)
+            self.debugViewer.showVariable(variables, True)
         elif scope == 0:
-            self.debugViewer.showVariable(variables, 0)
+            self.debugViewer.showVariable(variables, False)
             
     def __clientBreakConditionError(self, filename, lineno):
         """
@@ -1349,7 +1372,7 @@ class DebugUI(QObject):
         
         from .EditWatchpointDialog import EditWatchpointDialog
         dlg = EditWatchpointDialog(
-            (cond, temp, enabled, count, special), self)
+            (cond, temp, enabled, count, special), self.ui)
         if dlg.exec_() == QDialog.Accepted:
             cond, temp, enabled, count, special = dlg.getData()
             
@@ -1526,8 +1549,7 @@ class DebugUI(QObject):
             self.envHistory, self.exceptions, self.ui, 2,
             autoClearShell=self.autoClearShell)
         if dlg.exec_() == QDialog.Accepted:
-            argv, wd, env, exceptions, clearShell, clearHistories, console = \
-                dlg.getData()
+            argv, wd, env, exceptions, clearShell, console = dlg.getData()
             eraseCoverage = dlg.getCoverageData()
             
             if runProject:
@@ -1573,9 +1595,9 @@ class DebugUI(QObject):
             
             # This moves any previous occurrence of these arguments to the head
             # of the list.
-            self.setArgvHistory(argv, clearHistories)
-            self.setWdHistory(wd, clearHistories)
-            self.setEnvHistory(env, clearHistories)
+            self.setArgvHistory(argv)
+            self.setWdHistory(wd)
+            self.setEnvHistory(env)
             
             # Save the exception flags
             self.exceptions = exceptions
@@ -1608,7 +1630,17 @@ class DebugUI(QObject):
                     clientType=self.clientType)
                 
                 self.stopAct.setEnabled(True)
-            
+        
+        if dlg.clearHistories():
+            self.setArgvHistory("", clearHistories=True)
+            self.setWdHistory("", clearHistories=True)
+            self.setEnvHistory("", clearHistories=True)
+        elif dlg.historiesModified():
+            argvHistory, wdHistory, envHistory = dlg.getHistories()
+            self.setArgvHistory("", history=argvHistory)
+            self.setWdHistory("", history=wdHistory)
+            self.setEnvHistory("", history=envHistory)
+    
     def __profileScript(self):
         """
         Private slot to handle the profile script action.
@@ -1645,8 +1677,7 @@ class DebugUI(QObject):
             self.exceptions, self.ui, 3,
             autoClearShell=self.autoClearShell)
         if dlg.exec_() == QDialog.Accepted:
-            argv, wd, env, exceptions, clearShell, clearHistories, console = \
-                dlg.getData()
+            argv, wd, env, exceptions, clearShell, console = dlg.getData()
             eraseTimings = dlg.getProfilingData()
             
             if runProject:
@@ -1692,9 +1723,9 @@ class DebugUI(QObject):
             
             # This moves any previous occurrence of these arguments to the head
             # of the list.
-            self.setArgvHistory(argv, clearHistories)
-            self.setWdHistory(wd, clearHistories)
-            self.setEnvHistory(env, clearHistories)
+            self.setArgvHistory(argv)
+            self.setWdHistory(wd)
+            self.setEnvHistory(env)
             
             # Save the exception flags
             self.exceptions = exceptions
@@ -1727,7 +1758,17 @@ class DebugUI(QObject):
                     clientType=self.clientType)
                 
                 self.stopAct.setEnabled(True)
-            
+        
+        if dlg.clearHistories():
+            self.setArgvHistory("", clearHistories=True)
+            self.setWdHistory("", clearHistories=True)
+            self.setEnvHistory("", clearHistories=True)
+        elif dlg.historiesModified():
+            argvHistory, wdHistory, envHistory = dlg.getHistories()
+            self.setArgvHistory("", history=argvHistory)
+            self.setWdHistory("", history=wdHistory)
+            self.setEnvHistory("", history=envHistory)
+    
     def __runScript(self):
         """
         Private slot to handle the run script action.
@@ -1766,8 +1807,7 @@ class DebugUI(QObject):
             autoFork=self.forkAutomatically,
             forkChild=self.forkIntoChild)
         if dlg.exec_() == QDialog.Accepted:
-            argv, wd, env, exceptions, clearShell, clearHistories, console = \
-                dlg.getData()
+            argv, wd, env, exceptions, clearShell, console = dlg.getData()
             forkAutomatically, forkIntoChild = dlg.getRunData()
             
             if runProject:
@@ -1813,9 +1853,9 @@ class DebugUI(QObject):
             
             # This moves any previous occurrence of these arguments to the head
             # of the list.
-            self.setArgvHistory(argv, clearHistories)
-            self.setWdHistory(wd, clearHistories)
-            self.setEnvHistory(env, clearHistories)
+            self.setArgvHistory(argv)
+            self.setWdHistory(wd)
+            self.setEnvHistory(env)
             
             # Save the exception flags
             self.exceptions = exceptions
@@ -1850,6 +1890,16 @@ class DebugUI(QObject):
                 
                 self.stopAct.setEnabled(True)
         
+        if dlg.clearHistories():
+            self.setArgvHistory("", clearHistories=True)
+            self.setWdHistory("", clearHistories=True)
+            self.setEnvHistory("", clearHistories=True)
+        elif dlg.historiesModified():
+            argvHistory, wdHistory, envHistory = dlg.getHistories()
+            self.setArgvHistory("", history=argvHistory)
+            self.setWdHistory("", history=wdHistory)
+            self.setEnvHistory("", history=envHistory)
+    
     def __debugScript(self):
         """
         Private slot to handle the debug script action.
@@ -1887,8 +1937,7 @@ class DebugUI(QObject):
             autoClearShell=self.autoClearShell, autoContinue=self.autoContinue,
             autoFork=self.forkAutomatically, forkChild=self.forkIntoChild)
         if dlg.exec_() == QDialog.Accepted:
-            argv, wd, env, exceptions, clearShell, clearHistories, console = \
-                dlg.getData()
+            argv, wd, env, exceptions, clearShell, console = dlg.getData()
             tracePython, autoContinue, forkAutomatically, forkIntoChild = \
                 dlg.getDebugData()
             
@@ -1936,9 +1985,9 @@ class DebugUI(QObject):
             
             # This moves any previous occurrence of these arguments to the head
             # of the list.
-            self.setArgvHistory(argv, clearHistories)
-            self.setWdHistory(wd, clearHistories)
-            self.setEnvHistory(env, clearHistories)
+            self.setArgvHistory(argv)
+            self.setWdHistory(wd)
+            self.setEnvHistory(env)
             
             # Save the exception flags
             self.exceptions = exceptions
@@ -1991,6 +2040,16 @@ class DebugUI(QObject):
                 
                 self.stopAct.setEnabled(True)
         
+        if dlg.clearHistories():
+            self.setArgvHistory("", clearHistories=True)
+            self.setWdHistory("", clearHistories=True)
+            self.setEnvHistory("", clearHistories=True)
+        elif dlg.historiesModified():
+            argvHistory, wdHistory, envHistory = dlg.getHistories()
+            self.setArgvHistory("", history=argvHistory)
+            self.setWdHistory("", history=wdHistory)
+            self.setEnvHistory("", history=envHistory)
+    
     def __doRestart(self):
         """
         Private slot to handle the restart action to restart the last
@@ -2176,65 +2235,16 @@ class DebugUI(QObject):
         self.debugServer.remoteBreakpoint(
             aw.getFileName(), line, 1, None, 1)
         self.debugServer.remoteContinue()
-    
-    def __eval(self):
+
+    def __moveInstructionPointer(self):
         """
-        Private method to handle the Eval action.
+        Private method to move the instruction pointer to a different line.
         """
-        # Get the command line arguments.
-        if len(self.evalHistory) > 0:
-            curr = 0
-        else:
-            curr = -1
+        self.lastAction = 0
+        aw = self.viewmanager.activeWindow()
+        line = aw.getCursorPosition()[0] + 1
+        self.debugServer.remoteMoveIP(line)
 
-        arg, ok = QInputDialog.getItem(
-            self.ui,
-            self.tr("Evaluate"),
-            self.tr("Enter the statement to evaluate"),
-            self.evalHistory,
-            curr, True)
-
-        if ok:
-            if not arg:
-                return
-
-            # This moves any previous occurrence of this expression to the head
-            # of the list.
-            if arg in self.evalHistory:
-                self.evalHistory.remove(arg)
-            self.evalHistory.insert(0, arg)
-            
-            self.debugServer.remoteEval(arg)
-            
-    def __exec(self):
-        """
-        Private method to handle the Exec action.
-        """
-        # Get the command line arguments.
-        if len(self.execHistory) > 0:
-            curr = 0
-        else:
-            curr = -1
-
-        stmt, ok = QInputDialog.getItem(
-            self.ui,
-            self.tr("Execute"),
-            self.tr("Enter the statement to execute"),
-            self.execHistory,
-            curr, True)
-
-        if ok:
-            if not stmt:
-                return
-
-            # This moves any previous occurrence of this statement to the head
-            # of the list.
-            if stmt in self.execHistory:
-                self.execHistory.remove(stmt)
-            self.execHistory.insert(0, stmt)
-            
-            self.debugServer.remoteExec(stmt)
-            
     def __enterRemote(self):
         """
         Private method to update the user interface.
@@ -2244,7 +2254,6 @@ class DebugUI(QObject):
         """
         # Disable further debug commands from the user.
         self.debugActGrp.setEnabled(False)
-        self.debugActGrp2.setEnabled(False)
         
         self.viewmanager.unhighlight(True)
 
