@@ -390,6 +390,10 @@ class PluginRepositoryWidget(QWidget, Ui_PluginRepositoryDialog):
         self.__unstableItem = None
         self.__unknownItem = None
         
+        self.__newItems = 0
+        self.__updateLocalItems = 0
+        self.__updateRemoteItems = 0
+        
         self.downloadProgress.setValue(0)
         self.__doneMethod = None
         
@@ -429,6 +433,13 @@ class PluginRepositoryWidget(QWidget, Ui_PluginRepositoryDialog):
                     "No plugin repository file available.\nSelect Update.")
                  ])
             self.repositoryList.resizeColumnToContents(1)
+        
+        self.newLabel.setText(self.tr("New: <b>{0}</b>")
+                              .format(self.__newItems))
+        self.updateLocalLabel.setText(self.tr("Local Updates: <b>{0}</b>")
+                                      .format(self.__updateLocalItems))
+        self.updateRemoteLabel.setText(self.tr("Remote Updates: <b>{0}</b>")
+                                       .format(self.__updateRemoteItems))
     
     def __downloadFile(self, url, filename, doneMethod=None):
         """
@@ -510,7 +521,13 @@ class PluginRepositoryWidget(QWidget, Ui_PluginRepositoryDialog):
             return
         
         self.__downloadIODevice.open(QIODevice.WriteOnly)
-        self.__downloadIODevice.write(reply.readAll())
+        # read data in chunks
+        chunkSize = 64 * 1024 * 1024
+        while True:
+            data = reply.read(chunkSize)
+            if data is None or len(data) == 0:
+                break
+            self.__downloadIODevice.write(data)
         self.__downloadIODevice.close()
         if QFile.exists(self.__downloadFileName):
             QFile.remove(self.__downloadFileName)
@@ -596,12 +613,15 @@ class PluginRepositoryWidget(QWidget, Ui_PluginRepositoryDialog):
         elif updateStatus == PluginRepositoryWidget.PluginStatusNew:
             itm.setIcon(1, UI.PixmapCache.getIcon("download.png"))
             itm.setToolTip(1, self.tr("new download available"))
+            self.__newItems += 1
         elif updateStatus == PluginRepositoryWidget.PluginStatusLocalUpdate:
             itm.setIcon(1, UI.PixmapCache.getIcon("updateLocal.png"))
             itm.setToolTip(1, self.tr("update installable"))
+            self.__updateLocalItems += 1
         elif updateStatus == PluginRepositoryWidget.PluginStatusRemoteUpdate:
             itm.setIcon(1, UI.PixmapCache.getIcon("updateRemote.png"))
             itm.setToolTip(1, self.tr("updated download available"))
+            self.__updateRemoteItems += 1
     
     def __updateStatus(self, filename, version):
         """
