@@ -1,12 +1,17 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2011 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
 Module implementing the code style checker.
 """
 
+try:  # Only for Py2
+    import Queue as queue
+except ImportError:
+    import queue
+
 import sys
 import multiprocessing
 
@@ -18,7 +23,7 @@ pycodestyle.register_check(NamingStyleChecker, NamingStyleChecker.Codes)
 
 from DocStyleChecker import DocStyleChecker
 from MiscellaneousChecker import MiscellaneousChecker
-from McCabeChecker import McCabeChecker
+from ComplexityChecker import ComplexityChecker
 
 
 def initService():
@@ -74,7 +79,7 @@ class CodeStyleCheckerReport(pycodestyle.BaseReport):
         return code
 
 
-def extractLineFlags(line, startComment="#", endComment=""):
+def extractLineFlags(line, startComment="#", endComment="", flagsLine=False):
     """
     Function to extract flags starting and ending with '__' from a line
     comment.
@@ -82,20 +87,52 @@ def extractLineFlags(line, startComment="#", endComment=""):
     @param line line to extract flags from (string)
     @keyparam startComment string identifying the start of the comment (string)
     @keyparam endComment string identifying the end of a comment (string)
+    @keyparam flagsLine flag indicating to check for a flags only line (bool)
     @return list containing the extracted flags (list of strings)
     """
     flags = []
     
-    pos = line.rfind(startComment)
-    if pos >= 0:
-        comment = line[pos + len(startComment):].strip()
-        if endComment:
-            comment = comment.replace("endComment", "")
-        flags = [f.strip() for f in comment.split()
-                 if (f.startswith("__") and f.endswith("__"))]
+    if not flagsLine or (
+       flagsLine and line.strip().startswith(startComment)):
+        pos = line.rfind(startComment)
+        if pos >= 0:
+            comment = line[pos + len(startComment):].strip()
+            if endComment:
+                endPos = line.rfind(endComment)
+                if endPos >= 0:
+                    comment = comment[:endPos]
+            flags = [f.strip() for f in comment.split()
+                     if (f.startswith("__") and f.endswith("__"))]
     return flags
 
 
+def ignoreCode(code, lineFlags):
+    """
+    Function to check, if the given code should be ignored as per line flags.
+    
+    @param code error code to be checked
+    @type str
+    @param lineFlags list of line flags to check against
+    @type list of str
+    @return flag indicating to ignore the code
+    @rtype bool
+    """
+    if lineFlags:
+        
+        if "__IGNORE_WARNING__" in lineFlags:
+            # ignore all warning codes
+            return True
+        
+        for flag in lineFlags:
+            # check individual warning code
+            if flag.startswith("__IGNORE_WARNING_"):
+                ignoredCode = flag[2:-2].rsplit("_", 1)[-1]
+                if code.startswith(ignoredCode):
+                    return True
+    
+    return False
+
+
 def codeStyleCheck(filename, source, args):
     """
     Do the code style check and/ or fix found errors.
@@ -114,22 +151,31 @@ def codeStyleCheck(filename, source, args):
     return __checkCodeStyle(filename, source, args)
 
 
-def codeStyleBatchCheck(argumentsList, send, fx, cancelled):
+def codeStyleBatchCheck(argumentsList, send, fx, cancelled, maxProcesses=0):
     """
     Module function to check code style for a batch of files.
     
     @param argumentsList list of arguments tuples as given for codeStyleCheck
-    @param send reference to send function (function)
-    @param fx registered service name (string)
+    @type list
+    @param send reference to send function
+    @type func
+    @param fx registered service name
+    @type str
     @param cancelled reference to function checking for a cancellation
-        (function)
+    @type func
+    @param maxProcesses number of processes to be used
+    @type int
     """
-    try:
-        NumberOfProcesses = multiprocessing.cpu_count()
-        if NumberOfProcesses >= 1:
-            NumberOfProcesses -= 1
-    except NotImplementedError:
-        NumberOfProcesses = 1
+    if maxProcesses == 0:
+        # determine based on CPU count
+        try:
+            NumberOfProcesses = multiprocessing.cpu_count()
+            if NumberOfProcesses >= 1:
+                NumberOfProcesses -= 1
+        except NotImplementedError:
+            NumberOfProcesses = 1
+    else:
+        NumberOfProcesses = maxProcesses
 
     # Create queues
     taskQueue = multiprocessing.Queue()
@@ -148,11 +194,25 @@ def codeStyleBatchCheck(argumentsList, send, fx, cancelled):
     # Get and send results
     endIndex = len(argumentsList) - initialTasks
     for i in range(len(argumentsList)):
-        filename, result = doneQueue.get()
-        send(fx, filename, result)
-        if cancelled():
+        resultSent = False
+        wasCancelled = False
+        
+        while not resultSent:
+            try:
+                # get result (waiting max. 3 seconds and send it to frontend
+                filename, result = doneQueue.get(timeout=3)
+                send(fx, filename, result)
+                resultSent = True
+            except queue.Empty:
+                # ignore empty queue, just carry on
+                if cancelled():
+                    wasCancelled = True
+                    break
+        
+        if wasCancelled or cancelled():
             # just exit the loop ignoring the results of queued tasks
             break
+        
         if i < endIndex:
             taskQueue.put(argumentsList[i + initialTasks])
 
@@ -161,16 +221,16 @@ def codeStyleBatchCheck(argumentsList, send, fx, cancelled):
         taskQueue.put('STOP')
 
 
-def worker(input, output):
+def worker(inputQueue, outputQueue):
     """
     Module function acting as the parallel worker for the style check.
     
-    @param input input queue (multiprocessing.Queue)
-    @param output output queue (multiprocessing.Queue)
+    @param inputQueue input queue (multiprocessing.Queue)
+    @param outputQueue output queue (multiprocessing.Queue)
     """
-    for filename, source, args in iter(input.get, 'STOP'):
+    for filename, source, args in iter(inputQueue.get, 'STOP'):
         result = __checkCodeStyle(filename, source, args)
-        output.put((filename, result))
+        outputQueue.put((filename, result))
 
 
 def __checkCodeStyle(filename, source, args):
@@ -183,14 +243,16 @@ def __checkCodeStyle(filename, source, args):
     @param args arguments used by the codeStyleCheck function (list of
         excludeMessages (str), includeMessages (str), repeatMessages
         (bool), fixCodes (str), noFixCodes (str), fixIssues (bool),
-        maxLineLength (int), hangClosing (bool), docType (str), errors
-        (list of str), eol (str), encoding (str), backup (bool))
-    @return tuple of stats (dict) and results (tuple for each found violation
-        of style (tuple of lineno (int), position (int), text (str), ignored
-            (bool), fixed (bool), autofixing (bool), fixedMsg (str)))
+        maxLineLength (int), hangClosing (bool), docType (str), dictionary
+        with arguments for the code complexity checker (dict), dictionary
+        with arguments for the miscellaneous checker (dict), errors (list
+        of str), eol (str), encoding (str), backup (bool))
+    @return tuple of statistics (dict) and results (tuple for each found
+        violation of style (tuple of lineno (int), position (int), text (str),
+        ignored (bool), fixed (bool), autofixing (bool), fixedMsg (str)))
     """
     (excludeMessages, includeMessages, repeatMessages, fixCodes, noFixCodes,
-     fixIssues, maxLineLength, hangClosing, docType, maxComplexity,
+     fixIssues, maxLineLength, hangClosing, docType, codeComplexityArgs,
      miscellaneousArgs, errors, eol, encoding, backup) = args
     
     stats = {}
@@ -252,12 +314,12 @@ def __checkCodeStyle(filename, source, args):
         stats.update(miscellaneousChecker.counters)
         errors += miscellaneousChecker.errors
         
-        # check code complexity iaw. McCabe
-        mccabeChecker = McCabeChecker(
-            source, filename, select, ignore, maxComplexity)
-        mccabeChecker.run()
-        stats.update(mccabeChecker.counters)
-        errors += mccabeChecker.errors
+        # check code complexity
+        complexityChecker = ComplexityChecker(
+            source, filename, select, ignore, codeComplexityArgs)
+        complexityChecker.run()
+        stats.update(complexityChecker.counters)
+        errors += complexityChecker.errors
     
     errorsDict = {}
     for fname, lineno, position, text in errors:
@@ -274,8 +336,12 @@ def __checkCodeStyle(filename, source, args):
             if source:
                 code = text[0]
                 lineFlags = extractLineFlags(source[lineno - 1].strip())
-                if "__IGNORE_WARNING__" not in lineFlags and \
-                        "__IGNORE_WARNING_{0}__".format(code) not in lineFlags:
+                try:
+                    lineFlags += extractLineFlags(source[lineno].strip(),
+                                                  flagsLine=True)
+                except IndexError:
+                    pass
+                if not ignoreCode(code, lineFlags):
                     if fixer:
                         res, msg, id_ = fixer.fixIssue(lineno, position, text)
                         if res == -1:
