@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2007 - 2016 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2007 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -17,6 +17,7 @@ from E5Gui.E5Application import e5App
 
 import Preferences
 from Preferences.Shortcuts import readShortcuts
+import UI.Info
 
 from VcsPlugins.vcsPySvn.SvnUtilities import getConfigPath, getServersPath
 
@@ -25,7 +26,7 @@ name = "PySvn Plugin"
 author = "Detlev Offenbach <detlev@die-offenbachs.de>"
 autoactivate = False
 deactivateable = True
-version = "6.2.0"
+version = UI.Info.VersionOnly
 pluginType = "version_control"
 pluginTypename = "PySvn"
 className = "VcsPySvnPlugin"
