@@ -9,12 +9,13 @@ Module implementing a grabber widget for a rectangular snapshot region.
 
 from __future__ import unicode_literals
 
-from PyQt5.QtCore import pyqtSignal, Qt, QRect, QPoint, QTimer, QLocale, \
-    qVersion
+from PyQt5.QtCore import pyqtSignal, Qt, QRect, QPoint, QTimer, QLocale
 from PyQt5.QtGui import QPixmap, QColor, QRegion, QPainter, QPalette, \
     QPaintEngine, QPen, QBrush
 from PyQt5.QtWidgets import QWidget, QApplication, QToolTip
 
+from Globals import qVersionTuple
+
 
 def drawRect(painter, rect, outline, fill=None):
     """
@@ -114,7 +115,7 @@ class SnapshotRegionGrabber(QWidget):
         self.__desktop = QApplication.desktop()
         x = self.__desktop.x()
         y = self.__desktop.y()
-        if qVersion() >= "5.0.0":
+        if qVersionTuple() >= (5, 0, 0):
             self.__pixmap = QApplication.screens()[0].grabWindow(
                 self.__desktop.winId(), x, y,
                 self.__desktop.width(), self.__desktop.height())
