@@ -11,9 +11,11 @@ from __future__ import unicode_literals
 
 import re
 
-from PyQt5.QtCore import qVersion, Qt, QRegExp
+from PyQt5.QtCore import Qt, QRegExp
 from PyQt5.QtWebEngineCore import QWebEngineUrlRequestInfo
 
+from Globals import qVersionTuple
+
 
 def toSecondLevelDomain(url):
     """
@@ -532,7 +534,7 @@ class AdBlockRule(object):
         match = (
             req.resourceType() ==
             QWebEngineUrlRequestInfo.ResourceTypeSubResource)
-        if qVersion() >= "5.7.0":
+        if qVersionTuple() >= (5, 7, 0):
             match = match or (
                 req.resourceType() ==
                 QWebEngineUrlRequestInfo.ResourceTypePluginResource)
