@@ -33,7 +33,7 @@ class SpeedDial(QObject):
     
     @signal pagesChanged() emitted after the list of pages changed
     @signal thumbnailLoaded(url, src) emitted after a thumbnail was loaded
-    @signal titleLoaded(url, title) emitted after a title was loaded
+    @signal pageTitleLoaded(url, title) emitted after a title was loaded
     @signal speedDialSaved() emitted after the speed dial data was saved
     """
     pagesChanged = pyqtSignal()
